@@ -231,7 +231,7 @@ gst_ogm_audio_parse_get_type (void)
   return ogm_audio_parse_type;
 }
 
-GType
+static GType
 gst_ogm_video_parse_get_type (void)
 {
   static GType ogm_video_parse_type = 0;
@@ -257,7 +257,7 @@ gst_ogm_video_parse_get_type (void)
   return ogm_video_parse_type;
 }
 
-GType
+static GType
 gst_ogm_text_parse_get_type (void)
 {
   static GType ogm_text_parse_type = 0;
@@ -577,6 +577,7 @@ gst_ogm_parse_stream_header (GstOgmParse * ogm, const guint8 * data, guint size)
     }
     case 'v':{
       guint32 fourcc;
+      gint time_unit;
 
       fourcc = GST_MAKE_FOURCC (ogm->hdr.subtype[0],
           ogm->hdr.subtype[1], ogm->hdr.subtype[2], ogm->hdr.subtype[3]);
@@ -601,10 +602,15 @@ gst_ogm_parse_stream_header (GstOgmParse * ogm, const guint8 * data, guint size)
           ogm->hdr.samples_per_unit, ogm->hdr.default_len,
           ogm->hdr.buffersize, ogm->hdr.bits_per_sample, caps);
 
+      /* GST_TYPE_FRACTION contains gint */
+      if (ogm->hdr.time_unit > G_MAXINT || ogm->hdr.time_unit < G_MININT)
+        GST_WARNING_OBJECT (ogm, "timeunit is out of range");
+
+      time_unit = (gint) CLAMP (ogm->hdr.time_unit, G_MININT, G_MAXINT);
       gst_caps_set_simple (caps,
           "width", G_TYPE_INT, ogm->hdr.s.video.width,
           "height", G_TYPE_INT, ogm->hdr.s.video.height,
-          "framerate", GST_TYPE_FRACTION, 10000000, ogm->hdr.time_unit, NULL);
+          "framerate", GST_TYPE_FRACTION, 10000000, time_unit, NULL);
       break;
     }
     case 't':{
@@ -658,6 +664,15 @@ gst_ogm_parse_stream_header (GstOgmParse * ogm, const guint8 * data, guint size)
       gst_pad_push_event (ogm->srcpad, event);
     }
     g_list_free (cached_events);
+
+    {
+      GstTagList *tags;
+
+      tags = gst_tag_list_new ();
+      gst_tag_list_add (tags, GST_TAG_MERGE_APPEND, GST_TAG_SUBTITLE_CODEC,
+          "Ogm", NULL);
+      gst_element_found_tags_for_pad (GST_ELEMENT (ogm), ogm->srcpad, tags);
+    }
   }
 
   gst_caps_unref (caps);
