@@ -301,6 +301,7 @@ struct _GstRingBuffer {
  * @activate: activate the thread that starts pulling and monitoring the
  * consumed segments in the device. Since 0.10.22
  * @commit: write samples into the ringbuffer
+ * @clear_all: clear the entire ringbuffer Since 0.10.24
  *
  * The vmethods that subclasses can override to implement the ringbuffer.
  */
@@ -327,8 +328,10 @@ struct _GstRingBufferClass {
                                 guchar * data, gint in_samples, 
                                 gint out_samples, gint * accum);
 
+  void         (*clear_all)    (GstRingBuffer * buf);
+
   /*< private >*/
-  gpointer _gst_reserved[GST_PADDING - 2];
+  gpointer _gst_reserved[GST_PADDING - 3];
 };
 
 GType gst_ring_buffer_get_type(void);
