@@ -106,6 +106,12 @@ static GstStaticCaps gst_video_scale_format_caps[] = {
       "bpp = 8, "
       "depth = 8, "
       "width = " GST_VIDEO_SIZE_RANGE ", "
+      "height = " GST_VIDEO_SIZE_RANGE ", " "framerate = " GST_VIDEO_FPS_RANGE),
+  GST_STATIC_CAPS ("video/x-raw-gray, "
+      "bpp = 16, "
+      "depth = 16, "
+      "endianness = BYTE_ORDER, "
+      "width = " GST_VIDEO_SIZE_RANGE ", "
       "height = " GST_VIDEO_SIZE_RANGE ", " "framerate = " GST_VIDEO_FPS_RANGE)
 };
 
@@ -131,7 +137,8 @@ enum
   GST_VIDEO_SCALE_YV12,
   GST_VIDEO_SCALE_RGB565,
   GST_VIDEO_SCALE_RGB555,
-  GST_VIDEO_SCALE_GRAY8
+  GST_VIDEO_SCALE_GRAY8,
+  GST_VIDEO_SCALE_GRAY16
 };
 
 #define GST_TYPE_VIDEO_SCALE_METHOD (gst_video_scale_method_get_type())
@@ -445,6 +452,10 @@ gst_video_scale_prepare_size (GstVideoScale * videoscale, gint format,
       img->stride = GST_ROUND_UP_4 (img->width);
       *size = img->stride * img->height;
       break;
+    case GST_VIDEO_SCALE_GRAY16:
+      img->stride = GST_ROUND_UP_4 (img->width * 2);
+      *size = img->stride * img->height;
+      break;
     case GST_VIDEO_SCALE_I420:
     case GST_VIDEO_SCALE_YV12:
     {
@@ -768,6 +779,9 @@ gst_video_scale_transform (GstBaseTransform * trans, GstBuffer * in,
         case GST_VIDEO_SCALE_GRAY8:
           vs_image_scale_nearest_Y (dest, src, videoscale->tmp_buf);
           break;
+        case GST_VIDEO_SCALE_GRAY16:
+          vs_image_scale_nearest_Y16 (dest, src, videoscale->tmp_buf);
+          break;
         case GST_VIDEO_SCALE_I420:
         case GST_VIDEO_SCALE_YV12:
           vs_image_scale_nearest_Y (dest, src, videoscale->tmp_buf);
@@ -814,6 +828,9 @@ gst_video_scale_transform (GstBaseTransform * trans, GstBuffer * in,
         case GST_VIDEO_SCALE_GRAY8:
           vs_image_scale_linear_Y (dest, src, videoscale->tmp_buf);
           break;
+        case GST_VIDEO_SCALE_GRAY16:
+          vs_image_scale_linear_Y16 (dest, src, videoscale->tmp_buf);
+          break;
         case GST_VIDEO_SCALE_I420:
         case GST_VIDEO_SCALE_YV12:
           vs_image_scale_linear_Y (dest, src, videoscale->tmp_buf);
@@ -860,6 +877,9 @@ gst_video_scale_transform (GstBaseTransform * trans, GstBuffer * in,
         case GST_VIDEO_SCALE_GRAY8:
           vs_image_scale_4tap_Y (dest, src, videoscale->tmp_buf);
           break;
+        case GST_VIDEO_SCALE_GRAY16:
+          vs_image_scale_4tap_Y16 (dest, src, videoscale->tmp_buf);
+          break;
         case GST_VIDEO_SCALE_I420:
         case GST_VIDEO_SCALE_YV12:
           vs_image_scale_4tap_Y (dest, src, videoscale->tmp_buf);
