@@ -34,12 +34,12 @@ static guint schanged_id = 0;
 #define SEGMENT_LEN 100
 #define UPDATE_INTERVAL 500
 
-gdouble prev_range = -1.0;
-GstClockTime prev_time = GST_CLOCK_TIME_NONE;
-gdouble cur_range;
-GstClockTime cur_time;
-GstClockTimeDiff diff;
-gdouble cur_speed = 1.0;
+static gdouble prev_range = -1.0;
+static GstClockTime prev_time = GST_CLOCK_TIME_NONE;
+static gdouble cur_range;
+static GstClockTime cur_time;
+static GstClockTimeDiff diff;
+static gdouble cur_speed = 1.0;
 
 typedef struct
 {
@@ -109,8 +109,8 @@ make_wav_pipeline (const gchar * location)
 
   gst_element_link (src, decoder);
 
-  setup_dynamic_link (decoder, "src", gst_element_get_pad (audiosink, "sink"),
-      NULL);
+  setup_dynamic_link (decoder, "src", gst_element_get_static_pad (audiosink,
+          "sink"), NULL);
 
   return pipeline;
 }
@@ -300,7 +300,7 @@ start_seek (GtkWidget * widget, GdkEventButton * event, gpointer user_data)
   }
 
   if (changed_id == 0) {
-    changed_id = gtk_signal_connect (GTK_OBJECT (hscale),
+    changed_id = g_signal_connect (GTK_OBJECT (hscale),
         "value_changed", G_CALLBACK (seek_cb), pipeline);
   }
 
@@ -364,9 +364,10 @@ stop_cb (GtkButton * button, gpointer data)
   gst_element_get_state (pipeline, &state, NULL, GST_CLOCK_TIME_NONE);
   if (state != GST_STATE_READY) {
     g_print ("READY pipeline\n");
-    gst_element_set_state (pipeline, GST_STATE_PAUSED);
     gst_element_set_state (pipeline, GST_STATE_READY);
+    /* position and speed return to their default values */
     gtk_adjustment_set_value (adjustment, 0.0);
+    gtk_adjustment_set_value (sadjustment, 1.0);
     g_source_remove (update_id);
   }
 }
@@ -406,7 +407,7 @@ bus_message (GstBus * bus, GstMessage * message, gpointer data)
       GST_DEBUG ("segment_done, doing next seek");
       if (!do_seek (hscale, FALSE, update_id == 0)) {
         if (changed_id == 0) {
-          changed_id = gtk_signal_connect (GTK_OBJECT (hscale),
+          changed_id = g_signal_connect (GTK_OBJECT (hscale),
               "value_changed", G_CALLBACK (seek_cb), pipeline);
         }
       }
@@ -507,19 +508,19 @@ main (int argc, char **argv)
   gtk_range_set_update_policy (GTK_RANGE (hscale), GTK_UPDATE_CONTINUOUS);
 
   sadjustment =
-      GTK_ADJUSTMENT (gtk_adjustment_new (1.0, 0.0, 5.0, 0.1, 1.0, 1.0));
+      GTK_ADJUSTMENT (gtk_adjustment_new (1.0, 0.0, 5.0, 0.1, 1.0, 0.0));
   shscale = gtk_hscale_new (sadjustment);
   gtk_scale_set_digits (GTK_SCALE (shscale), 2);
   gtk_range_set_update_policy (GTK_RANGE (shscale), GTK_UPDATE_CONTINUOUS);
 
-  schanged_id = gtk_signal_connect (GTK_OBJECT (shscale),
+  schanged_id = g_signal_connect (GTK_OBJECT (shscale),
       "value_changed", G_CALLBACK (speed_cb), pipeline);
 
-  gtk_signal_connect (GTK_OBJECT (hscale),
+  g_signal_connect (GTK_OBJECT (hscale),
       "button_press_event", G_CALLBACK (start_seek), pipeline);
-  gtk_signal_connect (GTK_OBJECT (hscale),
+  g_signal_connect (GTK_OBJECT (hscale),
       "button_release_event", G_CALLBACK (stop_seek), pipeline);
-  gtk_signal_connect (GTK_OBJECT (hscale),
+  g_signal_connect (GTK_OBJECT (hscale),
       "format_value", G_CALLBACK (format_value), pipeline);
 
   /* do the packing stuff ... */
